@@ -15,7 +15,7 @@ use alloc::vec::Vec;
 #[cfg(feature = "std")]
 use std::vec::Vec;
 
-use crate::Error;
+use crate::{Error, Incrementable};
 
 /// `Vec` indexed with externally managed generational indexes.
 ///
@@ -36,7 +36,7 @@ use crate::Error;
 /// If the generation has a maximum value (e.g. `u8::MAX`), then the maximum value
 /// should serve as a tombstone to indicate that the index cannot be used.
 /// Any generational index with the maximum generation should never
-/// be used for any method except [`set_gen`][UnmanagedGenVec::set_gen].
+/// be used for any method except [`set_next_gen`][UnmanagedGenVec::set_next_gen].
 ///
 /// # Type Parameters
 ///
@@ -47,11 +47,12 @@ use crate::Error;
 /// ## `G`
 ///
 /// `G` is the generation type. `G` is usually a type like [u16] or [u32].
-/// By default, G is a [u16].
+/// By default, G is a [usize].
 ///
 /// Generation types must implement:
 ///
 /// * [`PartialOrd`]
+/// * [`Default`]
 ///
 /// The range of values for `G` determines how many generations a single index
 /// can be used. Assume a [u8] is used and a single index is allocated and
@@ -95,7 +96,7 @@ use crate::Error;
 /// * `Into<(I, G)> for GenIndex`
 #[derive(Clone, Debug, Hash, PartialEq, Eq, PartialOrd, Ord)]
 #[allow(clippy::module_name_repetitions)]
-pub struct UnmanagedGenVec<T, G = u16, I = usize, GenIndex = (I, G)> {
+pub struct UnmanagedGenVec<T, G = usize, I = usize, GenIndex = (I, G)> {
     inner: Vec<(G, T)>,
     index_ty: PhantomData<I>,
     gen_index_ty: PhantomData<GenIndex>,
@@ -130,6 +131,7 @@ impl<T, G, I, GenIndex> UnmanagedGenVec<T, G, I, GenIndex> {
     ///
     /// See [`Vec::len`] for additional information.
     #[must_use]
+    #[inline]
     pub fn len(&self) -> usize {
         self.inner.len()
     }
@@ -138,6 +140,7 @@ impl<T, G, I, GenIndex> UnmanagedGenVec<T, G, I, GenIndex> {
     ///
     /// See [`Vec::is_empty`] for additional information.
     #[must_use]
+    #[inline]
     pub fn is_empty(&self) -> bool {
         self.inner.is_empty()
     }
@@ -146,6 +149,7 @@ impl<T, G, I, GenIndex> UnmanagedGenVec<T, G, I, GenIndex> {
     ///
     /// See [`Vec::capacity`] for additional information.
     #[must_use]
+    #[inline]
     pub fn capacity(&self) -> usize {
         self.inner.capacity()
     }
@@ -153,6 +157,7 @@ impl<T, G, I, GenIndex> UnmanagedGenVec<T, G, I, GenIndex> {
     /// Reserves additional capacity of the inner [`Vec`].
     ///
     /// See [`Vec::reserve`] for additional information.
+    #[inline]
     pub fn reserve(&mut self, additional: usize) {
         self.inner.reserve(additional);
     }
@@ -160,6 +165,7 @@ impl<T, G, I, GenIndex> UnmanagedGenVec<T, G, I, GenIndex> {
     /// Reserves additional capacity of the inner [`Vec`].
     ///
     /// See [`Vec::reserve_exact`] for additional information.
+    #[inline]
     pub fn reserve_exact(&mut self, additional: usize) {
         self.inner.reserve_exact(additional);
     }
@@ -176,14 +182,27 @@ impl<T, G, I, GenIndex> Default for UnmanagedGenVec<T, G, I, GenIndex> {
 }
 
 impl<T, G, I, GenIndex> UnmanagedGenVec<T, G, I, GenIndex> {
+    /// Pushes the default generation and value to the end of the inner [`Vec`].
+    ///
+    /// See [`Vec::push`] for additional information.
+    #[inline]
+    pub fn push(&mut self, value: T)
+    where
+        G: Default,
+    {
+        self.inner.push((G::default(), value));
+    }
+
     /// Pushes a generation and value to the end of the inner [`Vec`].
     ///
     /// See [`Vec::push`] for additional information.
-    pub fn push(&mut self, generation: G, value: T) {
+    #[inline]
+    pub fn push_with_gen(&mut self, generation: G, value: T) {
         self.inner.push((generation, value));
     }
 
     /// Returns true if an element exists for the generational index.
+    #[inline]
     pub fn contains_index(&self, gen_index: GenIndex) -> bool
     where
         GenIndex: Into<(I, G)>,
@@ -202,7 +221,7 @@ impl<T, G, I, GenIndex> UnmanagedGenVec<T, G, I, GenIndex> {
     /// Errors are returned if:
     ///
     /// * the index is out of bounds
-    /// * the generation of the generational index is less than the generation associated with the element
+    /// * the generation of the generational index is not equal to the generation associated with the element
     pub fn get(&self, gen_index: GenIndex) -> Result<&T, Error>
     where
         GenIndex: Into<(I, G)>,
@@ -226,7 +245,7 @@ impl<T, G, I, GenIndex> UnmanagedGenVec<T, G, I, GenIndex> {
     /// Errors are returned if:
     ///
     /// * the index is out of bounds
-    /// * the generation of the generational index is less than the generation associated with the element
+    /// * the generation of the generational index is not equal to the generation associated with the element
     pub fn get_mut(&mut self, gen_index: GenIndex) -> Result<&mut T, Error>
     where
         GenIndex: Into<(I, G)>,
@@ -252,6 +271,7 @@ impl<T, G, I, GenIndex> UnmanagedGenVec<T, G, I, GenIndex> {
     /// # Safety
     ///
     /// There is no bounds check and no generation check performed. If the index is out of bounds, undefined behavior will occur.
+    #[inline]
     pub unsafe fn get_unchecked(&self, gen_index: GenIndex) -> &T
     where
         GenIndex: Into<(I, G)>,
@@ -268,6 +288,7 @@ impl<T, G, I, GenIndex> UnmanagedGenVec<T, G, I, GenIndex> {
     /// # Safety
     ///
     /// There is no bounds check and no generation check performed. If the index is out of bounds, undefined behavior will occur.
+    #[inline]
     pub unsafe fn get_unchecked_mut(&mut self, gen_index: GenIndex) -> &mut T
     where
         GenIndex: Into<(I, G)>,
@@ -280,6 +301,7 @@ impl<T, G, I, GenIndex> UnmanagedGenVec<T, G, I, GenIndex> {
     /// Returns the generation associated with the element at the index.
     ///
     /// Returns `None` if the index is out of bounds.
+    #[inline]
     pub fn get_gen(&self, index: I) -> Option<&G>
     where
         I: Into<usize>,
@@ -323,7 +345,8 @@ impl<T, G, I, GenIndex> UnmanagedGenVec<T, G, I, GenIndex> {
     ///
     /// * if the generation is greater than the current generation associated
     /// with the element. To increase the generation, a call to
-    /// [`set_gen`][UnmanagedGenVec::set_gen] must be called first.
+    /// [`set_next_gen`][UnmanagedGenVec::set_next_gen] must be called first.
+    #[inline]
     pub fn set(&mut self, gen_index: GenIndex, value: T) -> Result<(G, T), Error>
     where
         GenIndex: Into<(I, G)>,
@@ -337,7 +360,13 @@ impl<T, G, I, GenIndex> UnmanagedGenVec<T, G, I, GenIndex> {
     /// Sets or pushes the element at the index if the generation is equal to
     /// the existing generation associated with the element.
     ///
-    /// Returns the previous generation and the value for the element if replacing an existing value.
+    /// Returns the previous generation and the value for the element if
+    /// replacing an existing value.
+    ///
+    /// This method is a convenience method for the newest allocated
+    /// generational index. Either the newest allocated generationl index is
+    /// for an existing index or it is for the immediate next index if a
+    /// value were to be pushed to the `Vec`.
     ///
     /// # Errors
     ///
@@ -351,7 +380,7 @@ impl<T, G, I, GenIndex> UnmanagedGenVec<T, G, I, GenIndex> {
     /// * if the index is greater than the length of the inner vector
     /// * if the generation is greater than the current generation associated
     /// with the element. To increase the generation, a call to
-    /// [`set_gen`][UnmanagedGenVec::set_gen] must be called first.
+    /// [`set_next_gen`][UnmanagedGenVec::set_next_gen] must be called first.
     pub fn set_or_push(&mut self, gen_index: GenIndex, value: T) -> Result<Option<(G, T)>, Error>
     where
         GenIndex: Into<(I, G)>,
@@ -365,15 +394,16 @@ impl<T, G, I, GenIndex> UnmanagedGenVec<T, G, I, GenIndex> {
             Ordering::Less => self.internal_set(index, generation, value).map(Some),
             Ordering::Equal => {
                 debug_assert_eq!(index, self.len());
-                self.push(generation, value);
+                self.push_with_gen(generation, value);
                 Ok(None)
             }
             Ordering::Greater => Err(Error::index_out_of_bounds()),
         }
     }
 
-    /// Sets the generation for an index if the generation is greater than
-    /// the existing generation associated with the element.
+    /// Sets the next generation for an index. The `gen_index` parameter is
+    /// composed of the index and the next generation of the current generation
+    /// associated with the element.
     ///
     /// Returns the previous generation if successful.
     ///
@@ -384,10 +414,15 @@ impl<T, G, I, GenIndex> UnmanagedGenVec<T, G, I, GenIndex> {
     /// * the index is out of bounds
     /// * the generation is less than or equal to the existing generation associated with
     /// the element
-    pub fn set_gen(&mut self, gen_index: GenIndex) -> Result<G, Error>
+    ///
+    /// # Panics
+    ///
+    /// Panics if the generation is not the next generation after the existing
+    /// generation associated with the element.
+    pub fn set_next_gen(&mut self, gen_index: GenIndex) -> Result<G, Error>
     where
         GenIndex: Into<(I, G)>,
-        G: PartialOrd,
+        G: PartialOrd + Incrementable,
         I: Into<usize>,
     {
         let (index, generation) = gen_index.into();
@@ -396,6 +431,10 @@ impl<T, G, I, GenIndex> UnmanagedGenVec<T, G, I, GenIndex> {
             .get_mut(index.into())
             .ok_or_else(Error::index_out_of_bounds)?;
         if elem.0 < generation {
+            assert!(
+                elem.0.next().as_ref() == Some(&generation),
+                "generation is not the next generation of the current element"
+            );
             let prev_value = core::mem::replace(&mut elem.0, generation);
             Ok(prev_value)
         } else if elem.0 == generation {
@@ -460,77 +499,217 @@ mod tests {
     }
 
     #[test]
-    fn test_index_mut() {
-        let mut gen_vec = UnmanagedGenVec::<Value<u32>, u8>::default();
-
-        let index = gen_vec.len();
-        assert_eq!(index, 0);
-        let generation = 0;
-        gen_vec.push(generation, Value(0));
-        assert_eq!(gen_vec[(index, generation)], Value(0));
-
-        let value = &mut gen_vec[(index, generation)];
-        value.set(9);
-        assert_eq!(gen_vec[(index, generation)], Value(9));
+    fn test_contains_index_out_of_bounds() {
+        let gen_vec = UnmanagedGenVec::<Value<u32>, u8>::default();
+        assert!(!gen_vec.contains_index((0, 0)));
     }
 
     #[test]
-    #[should_panic(expected = "generation is greater than generation associated with element")]
-    fn test_set_with_generation_greater_than_existing() {
+    fn test_contains_index_generation_less_than_existing() {
         let mut gen_vec = UnmanagedGenVec::<Value<u32>, u8>::default();
-        gen_vec.push(0, Value(0u32));
-        gen_vec.set((0, 1), Value(1)).unwrap();
+        gen_vec.push_with_gen(1, Value(0));
+        assert!(!gen_vec.contains_index((0, 0)));
     }
 
     #[test]
-    fn test_set_with_index_out_of_bounds() {
+    fn test_contains_index_generation_greater_than_existing() {
+        let mut gen_vec = UnmanagedGenVec::<Value<u32>, u8>::default();
+        gen_vec.push_with_gen(1, Value(0));
+        assert!(!gen_vec.contains_index((2, 0)));
+    }
+
+    #[test]
+    fn test_get_index_out_of_bounds() {
+        let gen_vec = UnmanagedGenVec::<Value<u32>, u8>::default();
+        let err = gen_vec.get((0, 0)).unwrap_err();
+        assert!(err.is_index_out_of_bounds());
+    }
+
+    #[test]
+    fn test_get_generation_less_than_existing() {
+        let mut gen_vec = UnmanagedGenVec::<Value<u32>, u8>::default();
+        gen_vec.push_with_gen(1, Value(0));
+
+        let err = gen_vec.get((0, 0)).unwrap_err();
+        assert!(err.is_not_equal_generation_error());
+    }
+
+    #[test]
+    fn test_get_generation_greater_than_existing() {
+        let mut gen_vec = UnmanagedGenVec::<Value<u32>, u8>::default();
+        gen_vec.push_with_gen(1, Value(0));
+
+        let err = gen_vec.get((0, 2)).unwrap_err();
+        assert!(err.is_not_equal_generation_error());
+    }
+
+    #[test]
+    fn test_get_mut_index_out_of_bounds() {
+        let mut gen_vec = UnmanagedGenVec::<Value<u32>, u8>::default();
+        let err = gen_vec.get_mut((0, 0)).unwrap_err();
+        assert!(err.is_index_out_of_bounds());
+    }
+
+    #[test]
+    fn test_get_mut_generation_less_than_existing() {
+        let mut gen_vec = UnmanagedGenVec::<Value<u32>, u8>::default();
+        gen_vec.push_with_gen(1, Value(0));
+
+        let err = gen_vec.get_mut((0, 0)).unwrap_err();
+        assert!(err.is_not_equal_generation_error());
+    }
+
+    #[test]
+    fn test_get_mut_generation_greater_than_existing() {
+        let mut gen_vec = UnmanagedGenVec::<Value<u32>, u8>::default();
+        gen_vec.push_with_gen(1, Value(0));
+
+        let err = gen_vec.get_mut((0, 2)).unwrap_err();
+        assert!(err.is_not_equal_generation_error());
+    }
+
+    #[test]
+    fn test_get_gen_index_out_of_bounds() {
+        let gen_vec = UnmanagedGenVec::<Value<u32>, u8>::default();
+
+        assert_eq!(gen_vec.get_gen(0), None);
+    }
+
+    #[test]
+    fn test_set_index_out_of_bounds() {
         let mut gen_vec = UnmanagedGenVec::<Value<u32>, u8>::default();
         let err = gen_vec.set((0, 0), Value(1)).unwrap_err();
         assert!(err.is_index_out_of_bounds());
     }
 
     #[test]
-    #[should_panic(expected = "generation is greater than generation associated with element")]
-    fn test_set_or_push_with_generation_greater_than_existing() {
+    fn test_set_generation_less_than_existing() {
         let mut gen_vec = UnmanagedGenVec::<Value<u32>, u8>::default();
-        gen_vec.push(0, Value(0u32));
+        gen_vec.push_with_gen(1, Value(0));
+        let err = gen_vec.set((0, 0), Value(1)).unwrap_err();
+        assert!(err.is_generation_less_than_existing());
+    }
+
+    #[test]
+    #[should_panic(expected = "generation is greater than generation associated with element")]
+    fn test_set_generation_greater_than_existing() {
+        let mut gen_vec = UnmanagedGenVec::<Value<u32>, u8>::default();
+        gen_vec.push_with_gen(0, Value(0));
         gen_vec.set((0, 1), Value(1)).unwrap();
     }
 
     #[test]
-    fn test_set_or_push_with_index_out_of_bounds() {
+    fn test_set_or_push_index_out_of_bounds() {
         let mut gen_vec = UnmanagedGenVec::<Value<u32>, u8>::default();
         let err = gen_vec.set_or_push((1, 0), Value(1)).unwrap_err();
         assert!(err.is_index_out_of_bounds());
     }
 
     #[test]
-    #[should_panic(expected = "generation is not equal")]
-    fn test_index_access_with_wrong_generation() {
+    fn test_set_or_push_generation_less_than_existing() {
         let mut gen_vec = UnmanagedGenVec::<Value<u32>, u8>::default();
-        gen_vec.push(0, Value(0));
-        let _ = &gen_vec[(0, 1)];
+        gen_vec.push_with_gen(1, Value(0));
+        let err = gen_vec.set((0, 0), Value(1)).unwrap_err();
+        assert!(err.is_generation_less_than_existing());
     }
 
     #[test]
-    #[should_panic(expected = "generation is not equal")]
-    fn test_index_mut_access_with_wrong_generation() {
+    #[should_panic(expected = "generation is greater than generation associated with element")]
+    fn test_set_or_push_generation_greater_than_existing() {
         let mut gen_vec = UnmanagedGenVec::<Value<u32>, u8>::default();
-        gen_vec.push(0, Value(0));
-        let _ = &mut gen_vec[(0, 1)];
+        gen_vec.push_with_gen(0, Value(0));
+        gen_vec.set((0, 1), Value(1)).unwrap();
+    }
+
+    #[test]
+    fn test_set_next_gen_index_out_of_bounds() {
+        let mut gen_vec = UnmanagedGenVec::<Value<u32>, u8>::default();
+        let err = gen_vec.set_next_gen((0, 1)).unwrap_err();
+        assert!(err.is_index_out_of_bounds());
+    }
+
+    #[test]
+    fn test_set_next_gen_generation_already_equal() {
+        let mut gen_vec = UnmanagedGenVec::<Value<u32>, u8>::default();
+        gen_vec.push_with_gen(1, Value(0));
+        let err = gen_vec.set_next_gen((0, 1)).unwrap_err();
+        assert!(err.is_already_equal_generation_error());
+    }
+
+    #[test]
+    fn test_set_next_gen_generation_less_than_existing() {
+        let mut gen_vec = UnmanagedGenVec::<Value<u32>, u8>::default();
+        gen_vec.push_with_gen(2, Value(0));
+        let err = gen_vec.set_next_gen((0, 1)).unwrap_err();
+        assert!(err.is_generation_less_than_existing());
+    }
+
+    #[test]
+    #[should_panic(expected = "generation is not the next generation of the current element")]
+    fn test_set_next_gen_generation_greater_than_more_than_one_existing() {
+        let mut gen_vec = UnmanagedGenVec::<Value<u32>, u8>::default();
+        gen_vec.push_with_gen(1, Value(0));
+        let _ = gen_vec.set_next_gen((0, 3));
     }
 
     #[test]
     #[should_panic(expected = "index out of bounds")]
-    fn test_index_access_with_out_of_bounds_index() {
+    fn test_index_out_of_bounds_index() {
         let gen_vec = UnmanagedGenVec::<Value<u32>, u8>::default();
         let _ = gen_vec[(0, 0)];
     }
 
+    #[test]
+    #[should_panic(expected = "generation is not equal")]
+    fn test_index_generation_less_than_existing() {
+        let mut gen_vec = UnmanagedGenVec::<Value<u32>, u8>::default();
+        gen_vec.push_with_gen(1, Value(0));
+        let _ = &gen_vec[(0, 0)];
+    }
+
+    #[test]
+    #[should_panic(expected = "generation is not equal")]
+    fn test_index_generation_greater_than_existing() {
+        let mut gen_vec = UnmanagedGenVec::<Value<u32>, u8>::default();
+        gen_vec.push_with_gen(0, Value(0));
+        let _ = &gen_vec[(0, 1)];
+    }
+
     #[test]
     #[should_panic(expected = "index out of bounds")]
-    fn test_index_mut_access_with_out_of_bounds_index() {
+    fn test_index_mut_out_of_bounds_index() {
         let mut gen_vec = UnmanagedGenVec::<Value<u32>, u8>::default();
         let _ = &mut gen_vec[(0, 0)];
     }
+
+    #[test]
+    #[should_panic(expected = "generation is not equal")]
+    fn test_index_mut_generation_less_than_existing() {
+        let mut gen_vec = UnmanagedGenVec::<Value<u32>, u8>::default();
+        gen_vec.push_with_gen(1, Value(0));
+        let _ = &mut gen_vec[(0, 0)];
+    }
+
+    #[test]
+    #[should_panic(expected = "generation is not equal")]
+    fn test_index_mut_generation_greater_than_existing() {
+        let mut gen_vec = UnmanagedGenVec::<Value<u32>, u8>::default();
+        gen_vec.push_with_gen(0, Value(0));
+        let _ = &mut gen_vec[(0, 1)];
+    }
+
+    #[test]
+    fn test_index_mut() {
+        let mut gen_vec = UnmanagedGenVec::<Value<u32>, u8>::default();
+
+        let index = gen_vec.len();
+        assert_eq!(index, 0);
+        let generation = 0;
+        gen_vec.push_with_gen(generation, Value(0));
+        assert_eq!(gen_vec[(index, generation)], Value(0));
+
+        let value = &mut gen_vec[(index, generation)];
+        value.set(9);
+        assert_eq!(gen_vec[(index, generation)], Value(9));
+    }
 }
